package com.bwie.auth.service.impl;

import com.alibaba.fastjson.JSON;
import com.bwie.auth.feign.UserFeignService;
import com.bwie.auth.service.AuthService;
import com.bwie.common.constants.JwtConstants;
import com.bwie.common.constants.TokenConstants;
import com.bwie.common.domain.User;
import com.bwie.common.domain.request.LoginRequest;
import com.bwie.common.domain.response.TokenResponse;
import com.bwie.common.exception.Week3Exception;
import com.bwie.common.result.Result;
import com.bwie.common.utils.JwtUtils;
import com.bwie.common.utils.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.stereotype.Service;

import javax.servlet.http.HttpServletRequest;
import java.util.HashMap;
import java.util.Random;
import java.util.UUID;
import java.util.concurrent.TimeUnit;

/**
 * @Author：张腾
 * @Package：com.bwie.auth.service.impl
 * @Project：Zg6-Week3
 * @name：AuthServiceImpl
 * @Date：2024/8/6 9:43
 */
@Service
public class AuthServiceImpl implements AuthService {

    @Autowired
    private UserFeignService userFeignService;

    @Autowired
    private StringRedisTemplate redisTemplate;

    @Autowired
    private HttpServletRequest request;

    @Override
    public Result sendMsg(LoginRequest loginRequest) {
        if (StringUtils.isBlank(loginRequest.getUserName())){
            throw new Week3Exception("用户名/手机号不能为空");
        }

        Result<User> byName = userFeignService.findByName(loginRequest.getUserName());
        User user = byName.getData();
        if (null == user){
            throw new Week3Exception("该用户不存在请注册");
        }

        int smsCode = new Random().nextInt(9000)+1000;
        redisTemplate.opsForValue().set("code", JSON.toJSONString(smsCode),120, TimeUnit.SECONDS);
        return Result.success();
    }

    @Override
    public Result login(LoginRequest loginRequest) {
        if (StringUtils.isBlank(loginRequest.getUserName())){
            throw new Week3Exception("用户名/手机号不能为空");
        }

        Result<User> byName = userFeignService.findByName(loginRequest.getUserName());
        User user = byName.getData();
        if (null == user){
            throw new Week3Exception("该用户不存在请注册");
        }

        if (!redisTemplate.hasKey("code")){
            throw new Week3Exception("验证码过期请重新发送");
        }

        if (!loginRequest.getCode().equals(redisTemplate.opsForValue().get("code"))){
            throw new Week3Exception("验证码错误");
        }

        String userKey = UUID.randomUUID().toString().replaceAll("-", "");
        HashMap<String, Object> map = new HashMap<>();
        map.put(JwtConstants.USER_KEY,userKey);
        map.put(JwtConstants.DETAILS_USERNAME,user.getUserName());
        map.put(JwtConstants.DETAILS_USER_ID,user.getUserId());
        String token = JwtUtils.createToken(map);
        redisTemplate.opsForValue().set(TokenConstants.LOGIN_TOKEN_KEY+userKey,JSON.toJSONString(user),60,TimeUnit.MINUTES);
        return Result.success(TokenResponse
                .builder()
                .token(token)
                .time("60MIN")
                .build());
    }

    @Override
    public Result info() {
        String header = request.getHeader(TokenConstants.TOKEN);
        String userKey = JwtUtils.getUserKey(header);
        String uu = redisTemplate.opsForValue().get(TokenConstants.LOGIN_TOKEN_KEY + userKey);
        User user = JSON.parseObject(uu, User.class);
        return Result.success(user);
    }
}
