package com.bwie.firm.service.impl;

import com.alibaba.fastjson.JSON;
import com.bwie.common.constants.TokenConstants;
import com.bwie.common.domain.Task;
import com.bwie.common.domain.User;
import com.bwie.common.exception.Week3Exception;
import com.bwie.common.utils.JwtUtils;
import com.bwie.firm.mapper.TaskMapper;
import com.bwie.firm.service.FirmService;
import com.bwie.firm.service.TaskService;
import lombok.AllArgsConstructor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.kafka.core.KafkaTemplate;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.servlet.http.HttpServletRequest;
import java.util.List;

/**
 * @Author：张腾
 * @Package：com.bwie.firm.service.impl
 * @Project：Zg6-Week3
 * @name：TaskServiceImpl
 * @Date：2024/8/6 13:40
 */
@Service
@AllArgsConstructor
public class TaskServiceImpl implements TaskService {


    private TaskMapper taskMapper;

    private FirmService firmService;

    private StringRedisTemplate redisTemplate;

    private HttpServletRequest request;

    private KafkaTemplate<Object,Object> kafkaTemplate;

    @Override
    public Integer addTask(Task task) {
        String header = request.getHeader(TokenConstants.TOKEN);
        String userKey = JwtUtils.getUserKey(header);
        String s = redisTemplate.opsForValue().get(TokenConstants.LOGIN_TOKEN_KEY + userKey);
        User user = JSON.parseObject(s, User.class);
        task.setUserId(user.getUserId());
        Task task1 = taskMapper.selectByUserId(task.getUserId());
        if (null != task1){
            throw new Week3Exception("该岗位您已投递过，请耐心等待");
        }
        Integer i = taskMapper.addTask(task);
        firmService.updateCount(task.getFirmName());
        return i;
    }

    @Override
    public List<Task> findTask() {
        return taskMapper.findTask();
    }

    @Transactional
    @Override
    public void updateTaskStatus(Integer taskId, Integer status) {
        if (status == 1){
            kafkaTemplate.send("success",taskId);
        }
        if (status == 2){
            kafkaTemplate.send("fail",taskId);
        }
        taskMapper.updateTaskStatus(taskId,status);
    }
}
