package com.bwie.firm.sys;

import com.alibaba.fastjson.JSON;
import com.bwie.common.domain.response.FirmRes;
import com.bwie.firm.service.FirmService;
import lombok.extern.log4j.Log4j2;
import org.apache.commons.collections.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;

import java.util.List;
import java.util.UUID;

/**
 * @Author：张腾
 * @Package：com.bwie.firm.sys
 * @Project：Zg6-Week3
 * @name：Clocking
 * @Date：2024/8/6 11:31
 */
@Component
@Log4j2
public class Clocking {

    @Autowired
    private FirmService firmService;

    @Autowired
    private StringRedisTemplate redisTemplate;

    @Scheduled(cron = "0/50 * * * * ?")
    public void checkStatus(){
        log.info("定时任务:定时将失效的岗位状态修改为已过期，并且列表不展示,开始执行...");
        List<FirmRes> firmResList = firmService.selectByStatus();
        log.info("查询到开始时间距离当前时间大于10天的信息:");
        for (FirmRes firmRes : firmResList) {
            log.info(firmRes);
        }
        log.info("开始修改数据状态......");
        firmResList.forEach(item -> {
            firmService.updateStatus(item.getFirmId());
        });
        log.info("修改成功,如果某个企业需求全部过期，需要将同步Redis缓存中企业信息");
        firmResList.forEach(item -> {
           List<FirmRes> firmRes = firmService.selectById(item.getBossName());
           if (CollectionUtils.isEmpty(firmRes)){
               redisTemplate.opsForValue().set("编号为:"+item.getFirmId()+"==>"+UUID.randomUUID().toString(), JSON.toJSONString(item));
           }
        });
    }
}
